// The ANTiDISTRACTION device
// Created by Guershom Kitsa, Yena Lee, John Shen, and Nicole Zsoter

/* References for ultrasonic sensor code:
 *  https://www.dummies.com/computers/arduino/how-to-measure-distance-with-the-arduino/
 *  https://www.tutorialspoint.com/arduino/arduino_ultrasonic_sensor.htm
 */

/* Note about ultrasonic sensor inaccuracies:
   - The sensor is somewhat inaccurate for < 3cm, *occasionally* jumps above 10cm
    and rotates back when it shouldn't
       - Only comes up if something is very close to the sensor for multiple seconds
   - Sensor is *very* inaccurate for < 1cm, often jumps to ~2200cm if you're very close
 */

#include <Stepper.h>
int stepsPerRev = 2048; // adjust if needed by trial and error to equal a 360-degree turn
Stepper stepperMotor = Stepper(stepsPerRev, 8, 10, 9, 11);

int trigPin = 7; // trigger pin of sensor (sends out signals)
int echoPin = 6; // echo pin of sensor (receives signals)
long pulseDuration;

bool rotatedAway = false;
int attemptNumber = 1;

void setup() {
   stepperMotor.setSpeed(13); // in revs. per min.
     // highest value that works is 14 (with some difficulty)
     // (at higher speeds it doesn't change directions correctly)
     // different models may have different speeds; adjust by trial and error
   Serial.begin(9600);
   Serial.println("Hello\n");
}

void loop() {
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  // send a brief signal from the ultrasonic sensor
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10); // (10 microseconds is the shortest pulse length that pulseIn can read)
  digitalWrite(trigPin, LOW);

  // ultrasonic sensors require the use of the function pulseIn, which measures duration of one pulse
  // (if 2nd param is HIGH, result is time taken for signal to change from LOW to HIGH and back to LOW)
  //   (if LOW, for signal to change HIGH-LOW-HIGH)
  pulseDuration = pulseIn(echoPin, HIGH); // returns pulse length in microseconds

  // get distance in cm [speed of sound == 340 m/s == 29 microseconds per cm]
  long distance = pulseDuration / 29 / 2;
  Serial.print("object distance (cm): ");Serial.println(distance);
  
  /* Machine behaviours according to number of times you have approached the sensor:  
   * attempt 1: turn 180 degrees, turn back 1 second after you move away
   * attempt 2: same as attempt 1
   * attempt 3: turn 180 degrees, "shake" (rotate slightly back and forth) 5 times, then turn back after 1 second
   * attempts 4+: spin all the way around (3 revolutions without stopping)
  */
  
  if (distance < 10) {
    // check attempt # to determine behaviour
    Serial.print("====Attempt #");Serial.print(attemptNumber);Serial.println("====");
    if (attemptNumber <= 2) {
      // rotate away 180 degrees, but only if it's not already turned away
      if (rotatedAway == false) {
        Serial.println("---Too close! Rotating away---");
        rotatedAway = true;
        stepperMotor.step(stepsPerRev / 2);
        // note: sensor will not send/receive more signals until the motor is done rotating
        attemptNumber += 1;
      }
    }
    else if (attemptNumber == 3) {
      // rotate away, then "shake" (small rotations back and forth)
      Serial.println("---Rotate away then shake 5x---");
      if (rotatedAway == false) {
        // rotate if not already turned away
        rotatedAway = true;
        stepperMotor.step(stepsPerRev / 2);
      }
      // shake 5 times (whether already rotated away or not)
      int i = 0;
      while (i < 5) {
        stepperMotor.step(stepsPerRev / 15); // 360/15 = 24 degrees
        delay(200); // increase delay here or decrease motor speed in void setup() if the motor is not functioning properly
        stepperMotor.step(-stepsPerRev / 15); // negative numbers rotate the motor in the opposite direction        delay(100);
        delay(100); // this delay is effectively combined with the one at the end of void loop()
        i += 1;
      }
      attemptNumber += 1;
    }
    else if (attemptNumber >= 4) {
      // spin around in 3 full circles
      Serial.println("---Full turn 3x---");
      stepperMotor.step(stepsPerRev * 3);
      attemptNumber += 1;
    }
  }
  if (distance >= 10) {
    if (rotatedAway) {
      // rotate back to initial position if the machine had just turned 180 degrees
      // (after waiting for 1 second)
      Serial.println("---Far away! Rotating back---");
      delay(1000);
      stepperMotor.step(-stepsPerRev / 2);
      rotatedAway = false;
    }
  }

  delay(100); // make this delay shorter if the sensor is reacting too slowly to objects approaching

  // type "0" (zero) in the serial monitor to end serial communication
  if (Serial.available() > 0) {
    int num = Serial.parseInt();
    if (Serial.read() == '\n') {
      if (num == 0) {
        Serial.println("\nGoodbye");
        Serial.end();
      }
    }
  }

}
